import irc.bot
import requests
import time
from time import time as nowfunc
import twitchat.commands as commands
import json
from importlib import reload


cmd_obj = commands.Commands()


class TwitchBot(irc.bot.SingleServerIRCBot):
    def __init__(self, username, client_id, token, channel, keepalive=30):
        self.client_id = client_id
        self.token = token
        self.channel = '#' + channel
        self.headers = {'Client-ID': client_id,
                        'Accept': 'application/vnd.twitchtv.v5+json'}
        self.keepalive = keepalive
        self.reconnect = 1  # double every failed reconnection attmept

        # Get the channel id, we will need this for v5 API calls
        url = 'https://api.twitch.tv/kraken/users?login=' + channel
        r = requests.get(url, headers=self.headers).json()
        self.channel_id = r['users'][0]['_id']

        # Create IRC bot connection
        server = 'irc.chat.twitch.tv'
        port = 6667
        print('Connecting to ' + server + ' on port ' + str(port) + '...')
        irc.bot.SingleServerIRCBot.__init__(
                self, [(server, port, 'oauth:'+token)],
                username, username)

    def on_welcome(self, c, e):
        print('Joining ' + self.channel)

        # You must request specific capabilities before you can use them
        c.cap('REQ', ':twitch.tv/membership')
        c.cap('REQ', ':twitch.tv/tags')
        c.cap('REQ', ':twitch.tv/commands')
        c.join(self.channel)
        c.set_keepalive(self.keepalive)
        c.privmsg(self.channel, "I am here now :)")
        self.reconnect = 1

    def on_reconnect(self, c, e):
        time.sleep(self.reconnect)
        self.reconnect *= 2
        c.reconnect()

    def on_disconnect(self, c, e):
        time.sleep(self.reconnect)
        self.reconnect *= 2
        c.reconnect()

    def on_dccdisconnect(self, c, e):
        time.sleep(self.reconnect)
        self.reconnect *= 2
        c.reconnect()

    def on_pubmsg(self, c, e):
        """Tries to execute command if message begins with '!'"""
        if e.arguments[0][0] == "!":
            global cmd_obj
            if e.arguments[0] == "!reload":
                badges_tag = [dict['value'] for dict in e.tags
                              if dict['key'] == 'badges']
                badges_list = []
                if badges_tag[0]:
                    badges_list = badges_tag[0].split(",")
                badges_lists_list = [badge.split("/") for badge in badges_list
                                     if badge]

                badges = {badge_list[0]: badge_list[1] for badge_list
                          in badges_lists_list}
                for badge, value in badges.items():
                    if badge == "broadcaster" and value == "1":
                        global commands
                        commands = reload(commands)
                        cmd_obj = commands.Commands()
            else:
                self.exec_command(c, e, cmd_obj)

        else:
            self.send_timers(self.timers(), c)

    @commands.exec
    def exec_command(self, c, e, cmd_obj):
        """Check if command exists and checks for possible cooldown,
        then executes if possible"""
        return e, c, self, cmd_obj

    def timers(self):
        now = nowfunc()
        send = []
        with open('timers.json') as timerfile:
            timers = json.load(timerfile)

        for command, params in timers.items():
            params['messages'] += 1
            messages = params['messages']
            messages_threshold = params['messages_threshold']

            time = params['time']
            time_threshold = params['time_threshold']
            if (now - time > time_threshold and
               messages > messages_threshold):
                params['messages'] = 0
                params['time'] = now
                send.append(command)

            timers[command] = params

        with open('timers.json.bak', 'w') as timerbackup:
            json.dump(timers, timerbackup, indent=4)

        with open('timers.json', 'w') as timerfile:
            json.dump(timers, timerfile, indent=4)

        return send

    def send_timers(self, send, c):
        with open('timers.json') as timerfile:
            timers = json.load(timerfile)
        for command in send:
            name = timers[command]
            c.privmsg(self.channel,
                      name['post'])


def main():
    with open('settings.json') as settings_file:
        settings = json.load(settings_file)
    username = settings['username']
    client_id = settings['client_id']
    token = settings['token']
    channel = settings['channel']
    keepalive = settings['keepalive']

    bot = TwitchBot(username, client_id, token, channel, keepalive)
    bot.start()



template = "    @check_permissions\n    @check_cooldown(cooldown={cdwn})\n    def on_{cmdname}(self, e, msg, c, bot):\n        if len(msg) > {argnum}:\n            c.privmsg(bot.channel, \"{text}\"{formattext})\n\n"



"""Use these instances as placeholders to test commands.py"""


class E:
    def __init__(self):
        self.tags = [{'key': 'user-id', 'value': '1234TEST'}, {'key': 'badges',
                     'value': 'testbadge/1'}]


class C:
    def __init__(self):
        pass

    def privmsg(self, one, two):
        print(two)


class Bot:
    def __init__(self):
        self.channel = None


e = E()
c = C()
bot = Bot()



import requests


def get_uid(client_id, user):
    """Returns a user's user-id for permission checking"""
    headers = {"Client-ID": f"{client_id}",
               "Accept": "application/vnd.twitchtv.v5+json"}
    url = f"https://api.twitch.tv/kraken/users?login={user}"
    r = requests.get(url, headers=headers).json()
    return r['users'][0].get('_id')



import requests


def abstract(term):
    term = "+".join(term.split(" "))
    url = f"https://api.duckduckgo.com/?q={term}&format=json&" + \
        "no_redirect=1&t=twitch_chatbot"
    r = requests.get(url).json()
    abstract = r['Abstract'][:300] + "[...]"
    credit = f"Results by duckduckgo.com/?q={term}"
    source = r['AbstractURL']

    if abstract == "[...]":
        return "No abstract found"
    else:
        return f'"{abstract}" ; {credit} ; Source: {source}'


if __name__ == "__main__":
    term = input("What is your search term? ")
    print(abstract(term))



import json
from twitchat.permissions import permissions


def main():
    try:
        with open('settings.json') as settings_file:
            settings = json.load(settings_file)
    except FileNotFoundError:
        settings = {}

    try:
        open('timers.json').close()
    except FileNotFoundError:
        with open('timers.json', 'w') as timers_file:
            json.dump({}, timers_file, indent=4)

    try:
        open('extra_commands.py').close()
    except FileNotFoundError:
        open('extra_commands.py', 'w').close()

    try:
        open('permissions.json').close()
    except FileNotFoundError:
        with open('permissions.json', 'w') as permissions_file:
            json.dump(permissions, permissions_file, indent=4)

    settings['username'] = input("Username: ")
    settings['client_id'] = input("Client-ID: ")
    settings['token'] = input("token: ")
    settings['channel'] = input("channel: ").lower()
    settings['keepalive'] = 300
    with open('settings.json', 'w') as settings_file:
        json.dump(settings, settings_file, indent=4)



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['twitchat']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.24.0', 'irc>=18.0.0']

entry_points = \
{'console_scripts': ['twitchat = twitchat.twitchbot:main',
                     'twitchat-settings = twitchat.settings:main']}

setup(name='twitchat',
      version='1.2.11',
      description="A simple Twitch chat bot that gives you the option to run complicated commands, to an extent that other bots don't allow.",
      author='Fitti',
      author_email='dev@fitti.io',
      url='https://github.com/Fittiboy/python-twitch-bot',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.8',
     )



import functools
from datetime import datetime
import json

from twitchat.get_user_info import get_uid
from twitchat.duckduckgo_abstract import abstract

import extra_commands as ec


def exec(func):
    @functools.wraps(func)
    def exec_wrapper(*args, **kwargs):
        e, c, bot, cmd_obj = func(*args, **kwargs)
        msg = e.arguments[0].split(" ")
        e.arguments[0] = cmd_obj
        if len(msg[0]) == 1:
            return
        cmd = msg[0][1:]
        cmd_func_name = f"on_{cmd}".lower()
        method = getattr(cmd_obj, cmd_func_name, Commands.nothing)
        method(e, msg, c, bot)
    return exec_wrapper


class Commands:
    """Add all commands as methods to this class"""
    def __init__(self):
        self.cooldowns = {}
        for key, value in ec.__dict__.items():
            if len(key) > 3 and key[:3] == "on_":
                cd = ec.cooldowns[key]
                cooled = Commands.check_cooldown(cd)(value)
                permitted = Commands.check_permissions(cooled)
                self.__dict__[key] = permitted

    def nothing(self, *args):
        pass

    def check_cooldown(cooldown):
        """Use this decorator to add a cooldown to a command"""
        def cooldown_decorator(func):
            @functools.wraps(func)
            def cooldown_wrapper(*args, **kwargs):
                last_used = args[0].cooldowns.get(func.__name__)
                if last_used:
                    used_diff = datetime.now() - last_used
                    if used_diff.seconds < cooldown:
                        self, e, msg, c, bot = args
                        cmd = msg[0][1:]
                        current_cd = cooldown - used_diff.seconds
                        c.privmsg(bot.channel, f"{cmd} is still on" +
                                  f" a {current_cd} second cooldown!")
                        return
                args[0].cooldowns[func.__name__] = datetime.now()
                func(*args, **kwargs)
            return cooldown_wrapper
        return cooldown_decorator

    def check_permissions(func):
        """Use this decorator to add a permission check to a command"""
        @functools.wraps(func)
        def permissions_wrapper(*args, **kwargs):
            if len(args) == 4:
                args = list(args)
                args.insert(0, args[0].arguments[0])
            uid = [dict['value'] for dict in args[1].tags
                   if dict['key'] == 'user-id'][0]
            badges_tag = [dict['value'] for dict in args[1].tags
                          if dict['key'] == 'badges']
            badges_list = []
            if badges_tag[0]:
                badges_list = badges_tag[0].split(",")
            badges_lists_list = [badge.split("/") for badge in badges_list
                                 if badge]

            badges = {badge_list[0]: badge_list[1] for badge_list
                      in badges_lists_list}

            with open('permissions.json') as perms_file:
                perms = json.load(perms_file)
            func_perms = perms.get(func.__name__)

            if not func_perms:
                perms[func.__name__] = {
                    'all': "1",
                    'uids': [],
                    'badges': {},
                    'forbid': {
                        'all': "0",
                        'uids': [],
                        'badges': {}
                        }
                    }
                with open('permissions.json', 'w') as perms_file:
                    json.dump(perms, perms_file, indent=4)

            if func_perms:
                perm_uids = func_perms.get('uids')
                perm_badges = func_perms.get('badges')
                perm_all = func_perms.get('all')
                perm_forbid = func_perms.get('forbid')
                permitted = False
                forbidden = False
                if uid in perm_uids and uid not in perm_forbid['uids']:
                    permitted = True

                if uid in perm_forbid['uids']:
                    forbidden = True

                if perm_all == "1" and perm_forbid['all'] != "1":
                    permitted = True

                if perm_forbid['all'] == "1":
                    forbidden = True

                elif not forbidden:
                    for badge, value in badges.items():
                        print("Badge, value: ", badge, value)
                        if perm_badges.get(badge) == value:
                            permitted = True
                            break
                    for badge, value in badges.items():
                        if perm_forbid['badges'].get(badge) == value:
                            permitted = False
                            break

                if permitted and not forbidden:
                    func(*args, **kwargs)

        return permissions_wrapper

    # STOCK COMMANDS
    @check_permissions
    @check_cooldown(cooldown=0)
    def on_ping(self, e, msg, c, bot):
        """Checks if the bot is alive"""
        c.privmsg(bot.channel, 'pong')

    @check_permissions
    @check_cooldown(cooldown=10)
    def on_abstract(self, e, msg, c, bot):
        """Tries to find basic information on search term
        using the duckduckgo.com search engine"""
        if len(msg) > 1:
            term = " ".join(msg[1:])
            c.privmsg(bot.channel, abstract(term))

    @check_permissions
    @check_cooldown(cooldown=0)
    def on_channel(self, e, msg, c, bot):
        if len(msg) > 1:
            channel = msg[1]
            c.privmsg(bot.channel, f"Heading over to {channel}!")
            bot.channel = "#" + channel.lower()
            c.disconnect()
            bot.start()

    @check_permissions
    @check_cooldown(cooldown=0)
    def on_permissions(self, e, msg, c, bot):
        """Usage: !permissions add/remove/forbid command user/badge/all
        {username}/{badgename badge_value}
        you can check some possible badges at api.twitch.tv"""
        with open("permissions.json") as perms_file:
            perms = json.load(perms_file)

        if len(msg) >= 4:
            arf = msg[1]  # (a)dd (r)emove (f)orbid
            cmd = msg[2]  # (c)o(m)man(d)
            tp = msg[3]  # (t)y(p)e
        else:
            return

        cmd_perms = perms.get(f'on_{cmd}', None)

        if cmd_perms is None:
            cmd_perms = {
                'all': "0",
                'uids': [],
                'badges': {},
                'forbid': {
                    'all': "0",
                    'uids': [],
                    'badges': {}
                    }
                }

        if tp == "user" and len(msg) == 5:
            user = msg[4]

            try:
                uid = get_uid(bot.client_id, user)
            except Exception:
                return

            if arf == "add":
                if uid not in cmd_perms['uids']:
                    cmd_perms['uids'].append(uid)

                if uid in cmd_perms['forbid']['uids']:
                    cmd_perms['forbid']['uids'].remove(uid)

                c.privmsg(bot.channel, f"{user} can now use !{cmd}")

            elif arf == "remove":
                if uid in cmd_perms['uids']:
                    cmd_perms['uids'].remove(uid)

                c.privmsg(bot.channel, f"{user} can no longer use !{cmd}")

            elif arf == "forbid":
                if uid not in cmd_perms['forbid']['uids']:
                    cmd_perms['forbid']['uids'].append(uid)

                if uid in cmd_perms['uids']:
                    cmd_perms['uids'].remove(uid)

                c.privmsg(bot.channel, f"{user} is no longer allowed " +
                                       f"to use !{cmd}")

        elif tp == "badge" and len(msg) == 6:
            badge = msg[4]
            value = msg[5]

            if arf == "add":
                cmd_perms['badges'][badge] = value
                if cmd_perms['forbid']['badges'].get(badge) == value:
                    del cmd_perms['forbid']['badges'][badge]
                c.privmsg(bot.channel, f"Users with the {badge}/{value}" +
                                       f" badge can now use !{cmd}")

            elif arf == "remove":
                if cmd_perms['badges'].get(badge) == value:
                    del cmd_perms['badges'][badge]
                c.privmsg(bot.channel, f"Users with the {badge}/{value}" +
                                       f" badge can no longer use !{cmd}")

            elif arf == "forbid":
                cmd_perms['forbid']['badges'][badge] = value
                if cmd_perms['badges'].get(badge) == value:
                    del cmd_perms['badges'][badge]
                c.privmsg(bot.channel, f"Users with the {badge}/{value}" +
                                       " badge are not allowed to use " +
                                       f"!{cmd}")

        elif tp == "all":
            if arf == "add":
                cmd_perms['all'] = "1"
                cmd_perms['forbid']['all'] = "0"
                c.privmsg(bot.channel, "All users can now use" +
                                       f" !{cmd}")
            elif arf == "remove":
                cmd_perms['all'] = "0"
                c.privmsg(bot.channel, f"The !{cmd} command is no" +
                                       " longer available to all users")

            elif arf == "forbid":
                cmd_perms['forbid']['all'] = "1"
                cmd_perms['all'] = "0"
                c.privmsg(bot.channel, f"No user is allowed to use !{cmd}")

        perms[f'on_{cmd}'] = cmd_perms
        with open('permissions.json', 'w') as perms_file:
            json.dump(perms, perms_file, indent=4)
